#!/usr/bin/perl -w 

# LSB Test Execution Framework
# Results Page Module (tests_results.pl)
#
# Copyright (C) 2007 The Linux Foundation. All rights reserved.
#
# This program has been developed by ISP RAS for LF.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

use strict;
use Templates;
use Socket;
use URI::Escape;

autoflush_on();

my $boundary = '432432548930278320';

my $client_name = 'DTK Manager';

sub multipart_file
{
    my ($file_location, $filename, $field_name) = @_;

    my $content_type = 'application/octet-stream';
    
    unless (open FILE, $file_location.$filename) {
        print "Can't open ".$file_location.$filename."\n";
        print_footer();
        exit(1);
    }

    my $body_text = '';
    
    $body_text .= "\r\n";
    $body_text .= "Content-Disposition: form-data; name=\"$field_name\"; filename=\"$filename\"\r\n";
    $body_text .= "Content-Type: $content_type\r\n";
    #$body_text .= "Content-Transfer-Encoding: binary\r\n";
    $body_text .= "\r\n";

    # File content
    my $content_part;
    while ( read (FILE, $content_part, 1024*1024) ) {
        $body_text .= $content_part;
    }
    close FILE;

    $body_text .= "\r\n";
    $body_text .= "--$boundary";
    
    return $body_text;
}

sub multipart_text
{
    my ($name, $value) = @_;
    
    # Submit button data
    my $body_text = '';
    
    $body_text .= "\r\n";
    $body_text .= "Content-Disposition: form-data; name=\"$name\"\r\n";
    $body_text .= "\r\n";
    $body_text .= "$value\r\n";
    $body_text .= "--$boundary";
    
    return $body_text;
}

sub multipart_body
{
    my $filename = shift;
    my $directory = shift; # '/home/biga/local/work/cert/'; # with trailing slash.
    my $description = shift;
    my $purpose = shift;

    my $body_text = '';
    
    # Body start 
    $body_text .= "--$boundary";

    $body_text .= multipart_file($directory, $filename, 'attachment');
    $body_text .= multipart_text('submit', 'Hidden');
    $body_text .= multipart_text('purpose', $purpose);
    $body_text .= multipart_text('comments', $description);
    
    # Body End
    $body_text .= "--\r\n";
    
    return $body_text;
}

sub upload($)
{
    my $purpose = shift;
    
    print "HTTP/1.0 200 OK\n";
    print "Content-type: text/html\n\n";

    print_header("Test Results", "results");
    
    if ( !exists($_POST{'filename'} )
      || !exists($_POST{'directory'})
      || !exists($_POST{'comments'})
      || !exists($_POST{'secure_key'})
    ) {
        print "Not enough parameters";
        print_footer();
        exit;
    }
    
    my $filename = $_POST{'filename'};
    my $directory = $_POST{'directory'};    
    my $description = $_POST{'comments'};
    my $cert_url = $cert_sys_base.'upload.php?inadvance';
    
    $cert_url .= '&pcid='.$_POST{'pc_id'} if exists $_POST{'pc_id'};
    
    $cert_url = $_POST{'back_url'} if exists $_POST{'back_url'};
    
    $cert_sys_host = $_POST{'back_host'} if exists $_POST{'back_host'};
    
    # prepare the data
    my $body_text = &multipart_body( $filename, $directory, 
            $description, $purpose );

    my $content_length = length($body_text);

    # Bake cookies
    my %cookies = (
            'PHPSESSID' => $_POST{'secure_key'},
            'integration' => $client_name,
        );
    my @cookie_pairs = ();
    foreach my $key ( keys %cookies ) {
        my $cookie_pair = $key.'='.uri_escape($cookies{$key});
        push @cookie_pairs, $cookie_pair;
    }
    my $cookie_line = join('; ', @cookie_pairs);
    
    # Header settings
    my $host = $cert_sys_host;
    my $port = 80;
    my $pagename = $cert_url;
    my $header_content_type = "multipart/form-data";
    
    # Compile the header
    my $header = '';
    $header .= "POST $pagename HTTP/1.1\r\n";
    $header .= "Host: $host\r\n";
    $header .= "Content-Type: $header_content_type; boundary=$boundary\r\n";
    $header .= "Content-Length: $content_length\r\n";
    $header .= "Cookie: $cookie_line\r\n" if ( $cookie_line );
    $header .= "Connection: Close\r\n";
    $header .= "\r\n";
    
    #print "<pre>$header</pre>";
    
    # Entire message
    my $message = $header.$body_text;

    # start connecting
    socket ( SOCK, PF_INET, SOCK_STREAM, getprotobyname('tcp') );

    my $iaddr = inet_aton($host);
    my $paddr = sockaddr_in($port, $iaddr);

    my $result = connect(SOCK, $paddr);
    
    if ( ! $result ) {
    # Connection failed
        print '<p style="color:red">Error: Failed to connect to the Certification System'
                .' at '.$cert_sys_host.':80.</p>';
        print_footer();
    } else {
        print <<DATA;
<h1>Uploading test results</h1>
<p style="color:green;font-weight:bold">
 Please, wait while uploading the data to the Certification System...
</p>
<p>
 Time elapsed: 
  <span id="timer" style="font-family: monospace; color: blue;">00:00</span>
</p>
<br />
<table><tr><td><div style="width: 300px; height: 12px; border: 1px solid #004000;">
 <img src="images/progress.png" alt="progress: 0/0" 
   width="0" height="12" id="progress_bar" style="display: none;" />
</div>
</td><td>
<span id="progress_text" style="padding-left:30pt"></span>
</td></tr>
</table>
<br />
<p><span id="msg"><br/></span></p>

<script language="javascript" type="text/javascript">
// <![CDATA[
var elapsed_time = 0;
var timer;
function update_timer() {
    if ( !timer )  return;
	++elapsed_time;
	var elem = document.getElementById("timer");
	if (elem) {
		var m = Math.floor(elapsed_time / 60);
		var s = elapsed_time - m * 60;
		var tmp = "";
		tmp += (m < 10) ? "0" + m : m;
		tmp += ":";
		tmp += (s < 10) ? "0" + s : s;
		elem.innerHTML = tmp;
		timer = setTimeout("update_timer()", 1000);
	}
}

function set_progress(done, total) {
	var elem = document.getElementById("progress_bar");
	if (elem) {
		elem.style.display = ((done == 0) ? 'none' : '');
		if (done > total)
			done = total;
    	elem.alt = "progress: " + done + "/" + total;
		elem.width = Math.round(done * 300 / total);
	}
	var etxt = document.getElementById("progress_text");
	if (etxt) {
		if (done > total)
			done = total;
		etxt.innerHTML = "Uploaded: " + Math.floor(done/1024) + " of " + Math.floor(total/1024) +" Kbytes.";
	} 
}

function stopTimer() {
    clearTimeout(timer); 
    timer = 0;
}

timer = setTimeout("update_timer()", 1000);
// ]]>
</script>
DATA
	    print_footer(1);     # do not finish with </body></html> now!
    }

    # No bufferization!
    select SOCK; $| = 1;
    select STDOUT; $| = 1;

    # connected, let's post the data

    my $total_bytes = length($message);
    my $sent_bytes = 0;
    my $chunk_size = 1024*50;
    
    # Sending...
    $message =~ //; # reset search iterator
    for ( my $sent_bytes = 0; $sent_bytes < $total_bytes; ) { 
    # reading chunks of $chunk_size bytes
        my $chunk = substr($message, $sent_bytes, $chunk_size);

        print SOCK $chunk;
        
        $sent_bytes += (length($chunk) or last);
        
        # Progress bar increase
        print_javascript_progress ( $sent_bytes, $total_bytes );
        
        # DBG: sleep a bit
        #select(undef, undef, undef, 0.2);
    }

    ##### Data has been sent. Read the response. #####
    
    my $back_url = '';
    
    # TODO: F: Check the response code
    
    # Reading the response: looking for the "Location:" header field
    my $output = '';
    my $mode = 0;
    while (<SOCK>) {
        
        $output .= $_ if $mode==1;
        
        if ( /^Location:\s+(.*)$/ && $mode==0 ) {
            $back_url = $1;
            $back_url =~ s/\s+$//; # Trim trailing \r\n; 'chomp' doesn't work here!
        }
        
        if ( /^\s*$/ && $mode==0 ) {
        # End of header is marked by empty line.
            #last; # Do not read the rest of the response.
            $mode = 1;
        }
    }
    # close the connection
    close(SOCK);

    #if ( $output ) { print $output; exit; }
    
    if ( $back_url =~ m!^https?://! ) {
        # OK. Do nothing.
    }
    elsif ( $back_url =~ m!^/! ) {
        $back_url = 'http://'.$cert_sys_host.$back_url;
    }
    elsif ( $back_url =~ /^\w/ ) {
        $cert_url =~ s{/[^/]+$}{/$back_url};
        $back_url = 'http://'.$cert_sys_host.$cert_url;
    }
    elsif ( $back_url =~ /^\?/ ) {
        $cert_url =~ s{\?.+$}{$back_url};
        $back_url = 'http://'.$cert_sys_host.$cert_url;
    }
    elsif ( exists($_POST{'back_url_error'}) ) { 
        $back_url = 'http://'.$cert_sys_host.$_POST{'back_url_error'};
    }
    else {
        $back_url = '';
    }

    # Stop the timer on the page, print message.
    
    print <<OUT;
<script language="javascript" type="text/javascript">
// <![CDATA[
    stopTimer();
    var elem = document.getElementById("msg");
    if ( elem ) {
        elem.innerHTML = "<span style='color:green;font-weight:bold'>... done!</span>";
        elem.innerHTML += " &nbsp; ";
OUT
    if ( $back_url ne '' ) {
        print <<OUT;
        elem.innerHTML += "Press [<a href='$back_url'>Continue</a>]";
OUT
        }
    print <<OUT;
    }
// ]]>
</script>
OUT
    
    # Footer has already been printed, but without "</body></html>".
    print "</body>\n</html>\n";
    exit;
}

sub print_javascript_redirect
{
    my $back_url = shift;
    
    #print $back_url or
    print <<OUT;
<script language="javascript" type="text/javascript">
// <![CDATA[
    stopTimer();
    location.replace("$back_url");
// ]]>
</script>
OUT
}

sub print_javascript_progress
{
    my $sent_bytes = shift;
    my $total_bytes = shift;
    
    print <<OUT;
<script language="javascript" type="text/javascript">
// <![CDATA[
    set_progress($sent_bytes, $total_bytes);
// ]]>
</script>
OUT

}

sub get_test_info {
    my $subdir = shift;
    
    my $dir = $CONFIG{"RESULTS_DIR"}.'/'.$subdir;
    
    my $result;
    
    if ( open(FILE, $dir."/host_info/host_platform")) {
        $result->{'prod_arch'} = <FILE>;
        chomp $result->{'prod_arch'};
        $result->{'prod_arch'} =~ s/amd64/x86_64/;
    }

    if ( open(FILE, $dir."/host_info/host_os")) {
        my $distr = <FILE>;
        chomp $distr;
        
        if ( $distr =~ s/\(([^\)]+)\)// ) {
            if ( $1 ne $result->{'prod_arch'} ) {
                $result->{'prod_comment'} = $1;
            }
        }
        
        if ( $distr =~ s/\b(release)? *([0-9][0-9\.-\w]*)\b// ) {
            $result->{'prod_version'} = $2;
        }
        
        $distr =~ s/\s+$//; # Trim
        $distr =~ s/^\s+//; # Trim
        
        $result->{'prod_name'} = $distr;
    }
    
    if ( open(FILE, $dir."/host_info/lsb_release_all")) {
        while ( <FILE> ) {
            chomp;
            if ( /^Release:\s+(.*)$/ ) {
                $result->{'prod_version'} = $1;
            }
            if ( /^Codename:\s+(.*)$/ ) {
                next if ($1 eq 'n/a');
                $result->{'prod_comment'} = $1;
            }
        }
    }
    
    if ( open(FILE, $dir."/host_info/lsb_version")) {
        $result->{'lsb_version'} = <FILE>;
        chomp $result->{'lsb_version'};
    }
    
    return $result;
}

sub start()
{
    print "HTTP/1.0 200 OK\n";
    print "Content-type: text/html\n\n";

    print_header("Test Results", "results");

    my $file = $_GET{'file'};
    $file = $_POST{'file'} if (!$file);

    my $cert = $_GET{'cert'}; # Is this a good certification test result.
    $cert = $_POST{'cert'} if (!$cert);

    if ( index($file, $CONFIG{"RESULTS_DIR"}) != 0 ) {
        print "Wrong parameter: file (access forbidden)";
        print_footer();
        exit;
    }
    if ( ! -r $file ) {
        print "Wrong parameter: file (access denied)";
        print_footer();
        exit;
    }
    #
    my $get_credentials_page = "status.php";
    my $action_url = 'http://'.$cert_sys_host.$cert_sys_base.$get_credentials_page;
    
    my $base_url = 'http://'.$SERVER_PARAM{"HOST"};
    my $this_url = $base_url.'/cert_integration.pl'; 
    my $back_url = $this_url.'?external'; # Callback for file uploading
    my $client_url = $base_url.'/tests_results.pl'; # Return to it on success

    my $filename = '';
    my $file_location = '';
    
    if ( $file =~ m/^(.*\/)([^\/]*)$/ ) {
        $filename = $2;
        $file_location = $1; # with trailing slash.
    }
    else {
        print "Wrong parameter: file (can't split)";
        print_footer();
        exit;
    }
    
    my $filesize = -s $file_location.$filename;
    my $filelink = $base_url.'/get.pl'.$file_location.$filename;
    
    my $result;
    if ( $file_location =~ m/\/([^\/]+)\/+$/ ) {
        $result = get_test_info($1);
    }
    
    my $prod_kind = "distr";
    
    my $prod_name = ($result->{'prod_name'} or '');
    my $prod_version = ($result->{'prod_version'} or '');
    my $prod_arch = ($result->{'prod_arch'} or '');
    my $prod_comment = ($result->{'prod_comment'} or '');
    my $prod_url = ($result->{'prod_url'} or '');
    my $lsb_version = ($result->{'lsb_version'} or '');

    print <<DATA;
    <h1>Working with the Certification System</h1>
    <p>To apply for certification you should pass the following steps 
    in the Certification System.
    </p>
    <ul>
    <li>If you are not logged in you will be suggested
        to log in or create an account.</li>
    <li>If you are not a company member you will be suggested to register
        your company or join an existing one.</li>
    <li>Then you should register the product or chose a registered one.</li>
    <li>And after all you should start certification of the product
        by signing the Trademark License Agreement.</li>
    </ul>
    
    <p>All these steps are required to apply for certification.
    Once you click on the 'continue' link the Certification System page
    will be opened. Then follow the instructions.
    </p>
    <form method="post" name="dataform" action="$action_url">
    
    <input type="hidden" name="prod_kind" value="$prod_kind" />
    <input type="hidden" name="prod_name" value="$prod_name" />
    <input type="hidden" name="prod_version" value="$prod_version" />
    <input type="hidden" name="prod_comment" value="$prod_comment" />
    <input type="hidden" name="prod_arch" value="$prod_arch" />
    <input type="hidden" name="prod_url" value="$prod_url" />
    <input type="hidden" name="lsb_version" value="$lsb_version" />

    <input type="hidden" name="client" value="$client_name" />
    <input type="hidden" name="client_url" value="$client_url" />
    <input type="hidden" name="back_url" value="$back_url" />

    <input type="hidden" name="filename" value="$filename" />
    <input type="hidden" name="directory" value="$file_location" />
    <input type="hidden" name="filesize" value="$filesize" />
    <input type="hidden" name="filelink" value="$filelink" />
    <input type="hidden" name="cert" value="$cert" />

    <input type="hidden" name="integration" value="submit" />
    </form>
    <p>[<a href="#" onclick="document.dataform.submit();return false;"
        >Continue</a>]</p>
DATA
 
    print_footer();
    exit;
}

if ( exists($_GET{'start'}) && $_GET{'start'} ) {
    &start();
    exit;
}

if ( exists($_POST{'external'}) && $_POST{'external'} ) {
    upload('inadvance');
}

print "HTTP/1.0 200 OK\n";
print "Content-type: text/html\n\n";

print_header("Test Results", "results");
    
print <<DATA;
<p>This URL is used by the <a href="http://$cert_sys_host$cert_sys_base">Certification System</a> to interact with the DTK Manager.</p>
<p>If you see a message in the Certification System which warns you about 
sending your credentials to this address, then ensure that this is indeed the address
of the DTK Manager Web UI and submit the data.
</p>
<p>DTK Manager needs your credentials for uploading the file to the Certification System at $cert_sys_host.</p>
DATA

print_footer();
exit;
